/*
 *
 *  Copyright (C) 1997-2008 JDE Developers Team
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *  Authors : Sara Marugán <s.marugan@alumnos.urjc.es>
 *  Date: 5-05-2009
 */

/************************************************
 * jdec gazebo08 driver                           *
 ************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <pthread.h>
#include <unistd.h>
#include "libgazebo.h"
#include <stdlib.h>
#include "jde.h"
#include "varcolor.h"
#include "gazebo08.h"

/*
In gazebo08.h: 
GZ_LASER_MAX_RANGES
*/

#define NUM_LASER 180
#define NUM_BUMPERS 10
#define MAX_VEL 1000 /* mm/sec, hardware limit: 1800 */
#define MAX_RVEL 180 /* deg/sec, hardware limit: 360 */
/* SIF image size */
#define SIFNTSC_ROWS 240
#define SIFNTSC_COLUMNS 320


/** The driver's command thread cycle*/
#define gazebo08_COMMAND_CYCLE 100	/* ms */
/** The driver's main cycle*/
#define gazebo08_CYCLE 33 /*ms*/
/*
In libgazebo.h:
#define MAXCAM 8
*/
/** Max drivers name lenth*/
#define MAX_MODEL_ID	100 

/** libgazebo */
libgazebo *gz_lib = NULL;
lgz_camera *camera[MAXCAM];
lgz_stereo *stereo;

typedef struct gazebo08_camera_name
{
  /* tipo: 0 unassigned, 1 camera, 2 left disparity, 3 right disparity */
  int tipo;
  /* camara or stereo head gazebo08 id. NULL when not asigned */
  char name[MAX_MODEL_ID];	
} gc_name,* pgc_name;

/* The gazebo08 device names for jde laser, etc. For cameras, 
   color_name says which gazebo08 source feeds each jde camera */
char laser_name[MAX_MODEL_ID] = "";
char position_name[MAX_MODEL_ID] = "";
char motors_name[MAX_MODEL_ID] = "";
char ptz_name[MAX_MODEL_ID]="";
gc_name color_name[MAXCAM]; 

/* Variables put to 0.0 and no change during the execution */
float correcting_x = 0.; /* mm */
float correcting_y = 0.; /* mm */
float correcting_theta = 0.; /* deg */

/* gazebo08 Server and client */
int server_id = 0;
int client_id = 0;


/* Jde and drivers stuff declarations */
pthread_t gazebo08_th;
void *gazebo08_thread (void *not_used);

int laser_schema_id,
  encoders_schema_id,
  motors_schema_id,
  camera_schema_id[MAXCAM],
  ptmotors_schema_id,
  ptencoders_schema_id;

int gazebo08_encoders_run (int father, int *brothers, arbitration fn);
int gazebo08_encoders_stop ();
int gazebo08_motors_run (int father, int *brothers, arbitration fn);
int gazebo08_motors_stop ();
int gazebo08_camera0_run (int father, int *brothers, arbitration fn);
int gazebo08_camera0_stop ();
int gazebo08_camera1_run (int father, int *brothers, arbitration fn);
int gazebo08_camera1_stop ();
int gazebo08_camera2_run (int father, int *brothers, arbitration fn);
int gazebo08_camera2_stop ();
int gazebo08_camera3_run (int father, int *brothers, arbitration fn);
int gazebo08_camera3_stop ();
int gazebo08_camera4_run (int father, int *brothers, arbitration fn);
int gazebo08_camera4_stop ();
int gazebo08_camera5_run (int father, int *brothers, arbitration fn);
int gazebo08_camera5_stop ();
int gazebo08_camera6_run (int father, int *brothers, arbitration fn);
int gazebo08_camera6_stop ();
int gazebo08_camera7_run (int father, int *brothers, arbitration fn);
int gazebo08_camera7_stop ();
int gazebo08_laser_run (int father, int *brothers, arbitration fn);
int gazebo08_laser_stop ();
int gazebo08_ptmotors_run (int father, int *brothers, arbitration fn);
int gazebo08_ptmotors_stop ();
int gazebo08_ptencoders_run (int father, int *brothers, arbitration fn);
int gazebo08_ptencoders_stop ();

/* Functions to do the data transfer job */
void gazebo08_laser_callback ();
void gazebo08_camera_callback (int camnum);
void gazebo08_ptmotors_callback ();
void gazebo08_ptencoders_callback ();
void gazebo08_encoders_callback ();
void gazebo08_motors_callback ();


/** pthread state variable.*/
int state;
/** mutex for the thread of this driver.*/
pthread_mutex_t mymutex;
/** condition for the thread of this driver.*/
pthread_cond_t condition;


char driver_name[256] = "gazebo08";
int gazebo08_terminate_command = 0;

int serve_laser = 0, 
  serve_encoders = 0, 
  serve_motors = 0,
  serve_ptmotors = 0,
  serve_ptencoders = 0,
  serve_color[MAXCAM];

int laser_active = 0,
  encoders_active = 0, 
  motors_active = 0,
  ptmotors_active = 0,
  ptencoders_active = 0,
  color_active[MAXCAM];



/*Variables a exportar*/
char *colorA; /** 'colorA' schema image data*/
char *colorB; /** 'colorB' schema image data*/
char *colorC; /** 'colorC' schema image data*/
char *colorD;/** 'color' schema image data*/
Varcolor myA,myB,myC,myD; /* for varcolorA,varcolorB... */
float v; /* mm/s */
float w; /* deg/s*/
float jde_robot[5];
int jde_laser[NUM_LASER];
float pan_angle, tilt_angle;  /* degs */
float longitude; /* degs, pan angle */
float latitude; /* degs, tilt angle */
/** 'ptmotors' schema latitude and longitude speed control*/
float longitude_speed;
float latitude_speed;

/** gazebo08 pantilt max pan, degrees.*/
float max_pan = 54;
/** gazebo08 pantilt min pan, degrees.*/
float min_pan = -54;
/** gazebo08 pantilt max tilt, degrees.*/
float max_tilt = 44;
/** gazebo08 pantilt min tilt, degrees.*/
float min_tilt = -44;
/** Max longitude speed, degrees/sec. It is irrelevant as gazebo08 implements pantilt speed internally, so this parameters has no effect */
float max_longitude_speed = 20;
/** Max latitude speed, degrees/sec. It is irrelevant as gazebo08 implements pantilt speed internally, so this parameters has no effect */
float max_latitude_speed = 20;


int should_driver_stop()
{
  int i,shouldstop=1;

  if ((laser_active)||
      (encoders_active)||(motors_active)||
      (ptmotors_active)||(ptencoders_active))
    shouldstop=0;
  
  for(i=0;i<MAXCAM;i++)
    if (color_active[i]) shouldstop=0;

  if (shouldstop)
    {
      pthread_mutex_lock(&mymutex);
      state=slept;
      pthread_mutex_unlock(&mymutex);
    }
  return shouldstop;
}

int should_driver_restart()
{
  /* there is a race condition in the reading of the state variable
     here, but we can live with it */
  if (state==slept)
    {
      /* gazebo08 thread goes winner */
      pthread_mutex_lock(&mymutex);
      state=winner;
      pthread_mutex_unlock(&mymutex);
      pthread_cond_signal(&condition);
      return 1;
    }
  else return 0;
}


int
gazebo08_parseconf (char *configfile);

int
gazebo08_init (char *configfile)
{
  int i;
  int stereo_not_init_yet=1;
  
  puts ((char*)"Starting gazebo08 driver");
  fflush (stdout);
  
  for(i=0;i<MAXCAM;i++)
  {
      camera[i]=NULL;
      color_name[i].tipo=0;
      serve_color[i]=0;
      color_active[i]=0;
      camera_schema_id[i]=0;
  }

  /* Parse the configuration file */
  if (gazebo08_parseconf (configfile) == -1)
    {
      printf ((char*)"gazebo08: driver not initialized. Configuration file parsing error.\n");
      exit (-1);
    }

  /* Connection with gazebo08 simulator */
  printf ((char*)"Connecting to gazebo08 Simulator\n");

  // Create libgazebo object
  gz_lib= lgz_create();
  if(gz_lib==NULL){
      printf ((char*)"gazebo08: cannot access to libgazebo library.\n");
      exit (-1);
  }
  
  // Connect to the libgazebo server
  if(lgz_init(gz_lib,server_id,client_id)==-1){
      printf ((char*)"gazebo08: libgazebo init fail.\n");
      exit (-1);
  }
  
  if (serve_motors || serve_encoders)
  {
	  /// Open the Position interface
	  if(lgz_openPosIface(gz_lib,(char*)motors_name)==-1){
		printf ((char*)"gazebo08: openPosIface fail.\n");
	  }
  }

  if (serve_laser)
  {
	  /// Open the Laser interface
	  if(lgz_openLaserIface(gz_lib,(char*)laser_name)==-1){
		printf ((char*)"gazebo08: openLaserIface fail.\n");
	  }
  }

  if (serve_ptencoders || serve_ptmotors)
  {
	  /// Open the Position interface
	  if(lgz_openPtzIface(gz_lib,(char*)ptz_name)==-1){
		printf ((char*)"gazebo08: openPtzIface fail.\n");
	  }
  }

  for(i=0;i<MAXCAM;i++)
    {
      if (serve_color[i]) 
	{
	  if (color_name[i].tipo == 1)
	    {	  
		camera[i] = (lgz_camera*) malloc(sizeof(lgz_camera));
	        if(lgz_openCameraIface(gz_lib,color_name[i].name,camera[i])==-1){
		  	printf ((char*)"gazebo08: openCameraIface fail.\n");
		}
	    }
	  else if ((color_name[i].tipo >1)&&(stereo_not_init_yet))
	    /* only one stereo device in this driver. The first jde-camera inits the stereo gazebo device */
	    {
              stereo = (lgz_stereo*) malloc(sizeof(lgz_stereo));
	      if(lgz_openStereoIface(gz_lib,color_name[i].name,stereo)==-1){
		  	printf ((char*)"gazebo08: openStereoIface fail.\n");
	      }
	      stereo_not_init_yet=0;
	    }
	}
    }
 
  /* Set up the virtual schemas API */
  if (serve_motors)
    {
      all[num_schemas].id = (int *) &motors_schema_id;
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"motors");
      all[num_schemas].run = (runFn) gazebo08_motors_run;
      all[num_schemas].stop = (stopFn) gazebo08_motors_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport((char*)"motors",(char*)"id", &motors_schema_id);
      myexport((char*)"motors",(char*)"v", &v);
      myexport((char*)"motors",(char*)"w", &w);
      myexport((char*)"motors",(char*)"run", (void *) &gazebo08_motors_run);
      myexport((char*)"motors",(char*)"stop", (void *) &gazebo08_motors_stop);
      v=0; w=0;
    }
  
  if (serve_laser)
    {
      all[num_schemas].id = (int *) &laser_schema_id;
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"laser");
      all[num_schemas].run = (runFn) gazebo08_laser_run;
      all[num_schemas].stop = (stopFn) gazebo08_laser_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport ((char*)"laser",(char*)"id", &laser_schema_id);
      myexport ((char*)"laser",(char*)"laser", &jde_laser);
      myexport ((char*)"laser",(char*)"run", (void *) &gazebo08_laser_run);
      myexport ((char*)"laser",(char*)"stop", (void *) &gazebo08_laser_stop);
    }

  if (serve_ptmotors)
    {
      all[num_schemas].id = (int *) &ptmotors_schema_id;
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"ptmotors");
      all[num_schemas].run = (runFn) gazebo08_ptmotors_run;
      all[num_schemas].stop = (stopFn) gazebo08_ptmotors_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport((char*)"ptmotors",(char*)"id", &ptmotors_schema_id);
      myexport((char*)"ptmotors",(char*)"longitude",&longitude);
      myexport((char*)"ptmotors",(char*)"latitude",&latitude);
      myexport((char*)"ptmotors",(char*)"longitude_speed",&longitude_speed);
      myexport((char*)"ptmotors",(char*)"latitude_speed",&latitude_speed);
      myexport((char*)"ptmotors",(char*)"max_longitude", &max_pan);
      myexport((char*)"ptmotors",(char*)"min_longitude", &min_pan);
      myexport((char*)"ptmotors",(char*)"max_latitude", &max_tilt);
      myexport((char*)"ptmotors",(char*)"min_latitude", &min_tilt);
      myexport((char*)"ptmotors",(char*)"max_longitude_speed", &max_longitude_speed);
      myexport((char*)"ptmotors",(char*)"max_latitude_speed", &max_latitude_speed);
      myexport((char*)"ptmotors",(char*)"run",(void *)&gazebo08_ptmotors_run);
      myexport((char*)"ptmotors",(char*)"stop",(void *)&gazebo08_ptmotors_stop);
    }
  
  if (serve_ptencoders)
    {
      all[num_schemas].id = (int *) &ptencoders_schema_id;
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"ptencoders");
      all[num_schemas].run = (runFn) gazebo08_ptencoders_run;
      all[num_schemas].stop = (stopFn) gazebo08_ptencoders_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport ((char*)"ptencoders",(char*)"id", &ptencoders_schema_id);
      myexport ((char*)"ptencoders",(char*)"pan_angle", &pan_angle);
      myexport ((char*)"ptencoders",(char*)"tilt_angle", &tilt_angle);
      myexport((char*)"ptencoders",(char*)"run",(void *)&gazebo08_ptencoders_run);
      myexport((char*)"ptencoders",(char*)"stop",(void *)&gazebo08_ptencoders_stop);
      }

  if (serve_encoders)
    {
      all[num_schemas].id = (int *) &encoders_schema_id;
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"encoders");
      all[num_schemas].run = (runFn) gazebo08_encoders_run;
      all[num_schemas].stop = (stopFn) gazebo08_encoders_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport ((char*)"encoders",(char*)"id", &encoders_schema_id);
      myexport ((char*)"encoders",(char*)"jde_robot", &jde_robot);
      myexport ((char*)"encoders",(char*)"run", (void *) &gazebo08_encoders_run);
      myexport ((char*)"encoders",(char*)"stop", (void *) &gazebo08_encoders_stop);
    }

  if (serve_color[0])
    {
      colorA = (char*)malloc(SIFNTSC_COLUMNS*SIFNTSC_ROWS*3*sizeof(char));
      all[num_schemas].id = (int *) &camera_schema_id[0];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"colorA");
      all[num_schemas].run = (runFn) gazebo08_camera0_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera0_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport((char*)"colorA",(char*)"id",&camera_schema_id[0]);
      myexport((char*)"colorA",(char*)"colorA",&colorA);
      myexport((char*)"colorA",(char*)"run",(void *)gazebo08_camera0_run);
      myexport((char*)"colorA",(char*)"stop",(void *)gazebo08_camera0_stop);
    }

  if (serve_color[1])
    {
      colorB = (char*)malloc(SIFNTSC_COLUMNS*SIFNTSC_ROWS*3*sizeof(char));
      all[num_schemas].id = (int *) &camera_schema_id[1];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"colorB");
      all[num_schemas].run = (runFn) gazebo08_camera1_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera1_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport((char*)"colorB",(char*)"id",&camera_schema_id[1]);
      myexport((char*)"colorB",(char*)"colorB",&colorB);
      myexport((char*)"colorB",(char*)"run",(void *)gazebo08_camera1_run);
      myexport((char*)"colorB",(char*)"stop",(void *)gazebo08_camera1_stop);
    }
  if (serve_color[2])
    {
      colorC = (char*)malloc(SIFNTSC_COLUMNS*SIFNTSC_ROWS*3*sizeof(char));
      all[num_schemas].id = (int *) &camera_schema_id[2];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"colorC");
      all[num_schemas].run = (runFn) gazebo08_camera2_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera2_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport((char*)"colorC",(char*)"id",&camera_schema_id[2]);
      myexport((char*)"colorC",(char*)"colorC",&colorC);
      myexport((char*)"colorC",(char*)"run",(void *)gazebo08_camera2_run);
      myexport((char*)"colorC",(char*)"stop",(void *)gazebo08_camera2_stop);
    }
  if (serve_color[3])
    {
      colorD = (char*)malloc(SIFNTSC_COLUMNS*SIFNTSC_ROWS*3*sizeof(char));
      all[num_schemas].id = (int *) &camera_schema_id[3];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy (all[num_schemas].name,(char*)"colorD");
      all[num_schemas].run = (runFn) gazebo08_camera3_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera3_stop;
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name,num_schemas);
      num_schemas++;
      myexport((char*)"colorD",(char*)"id",&camera_schema_id[3]);
      myexport((char*)"colorD",(char*)"colorD",&colorD);
      myexport((char*)"colorD",(char*)"run",(void *)gazebo08_camera3_run);
      myexport((char*)"colorD",(char*)"stop",(void *)gazebo08_camera3_stop);
    }

  
  if (serve_color[4])
    {
      myA.img = NULL;
      myA.clock = 0;
      /* the gazebo08 device must be opened before this operation. The image size 
	 of the jde varcolor image is taken from gazebo08 device */
      if (color_name[4].tipo == 1){	  
	  myA.width=camera[4]->width;
	  myA.height=camera[4]->height;
      }
      else if (color_name[4].tipo > 1){
	  myA.width=stereo->width;
	  myA.height=stereo->height;
      }
      myA.img = (char*)malloc(myA.width*myA.height*3*sizeof(char));
      //printf("gazebo: myA w=%d h=%d\n",myA.width,myA.height);
      all[num_schemas].id = (int *) &camera_schema_id[4];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy(all[num_schemas].name,(char*)"varcolorA");
      all[num_schemas].run = (runFn) gazebo08_camera4_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera4_stop;     
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;     
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name, *(all[num_schemas].id));
      num_schemas++;
      myexport((char*)"varcolorA",(char*)"id",&camera_schema_id[4]);
      myexport((char*)"varcolorA",(char*)"varcolorA",&myA);
      myexport((char*)"varcolorA",(char*)"run",(void *)gazebo08_camera4_run);
      myexport((char*)"varcolorA",(char*)"stop",(void *)gazebo08_camera4_stop);
    }

  if (serve_color[5])
    {
      myB.img = NULL;
      myB.clock = 0;
      /* the gazebo08 device must be opened before this operation. The image size 
	 of the jde varcolor image is taken from gazebo08 device */
      if (color_name[5].tipo == 1){	  
	  myB.width=camera[5]->width;
	  myB.height=camera[5]->height;
      }
      else if (color_name[5].tipo > 1){
	  myB.width=stereo->width;
	  myB.height=stereo->height;
      }
      myB.img = (char*)malloc(myB.width*myB.height*3*sizeof(char));
      all[num_schemas].id = (int *) &camera_schema_id[5];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy(all[num_schemas].name,(char*)"varcolorB");
      all[num_schemas].run = (runFn) gazebo08_camera5_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera5_stop;     
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;     
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name, *(all[num_schemas].id));
      num_schemas++;
      myexport((char*)"varcolorB",(char*)"id",&camera_schema_id[5]);
      myexport((char*)"varcolorB",(char*)"varcolorB",&myB);
      myexport((char*)"varcolorB",(char*)"run",(void *)gazebo08_camera5_run);
      myexport((char*)"varcolorB",(char*)"stop",(void *)gazebo08_camera5_stop);
    }

  if (serve_color[6])
    {
      myC.img = NULL;
      myC.clock = 0;
      if (color_name[6].tipo == 1){	  
	  myC.width=camera[6]->width;
	  myC.height=camera[6]->height;
      }
      else if (color_name[6].tipo > 1){
	  myC.width=stereo->width;
	  myC.height=stereo->height;
      }
      myC.img = (char*)malloc(myC.width*myC.height*3*sizeof(char));
      all[num_schemas].id = (int *) &camera_schema_id[6];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy(all[num_schemas].name,(char*)"varcolorC");
      all[num_schemas].run = (runFn) gazebo08_camera6_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera6_stop;     
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;     
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name, *(all[num_schemas].id));
      num_schemas++;
      myexport((char*)"varcolorC",(char*)"id",&camera_schema_id[6]);
      myexport((char*)"varcolorC",(char*)"varcolorC",&myC);
      myexport((char*)"varcolorC",(char*)"run",(void *)gazebo08_camera6_run);
      myexport((char*)"varcolorC",(char*)"stop",(void *)gazebo08_camera6_stop);
    }

  if (serve_color[7])
    {
      myD.img = NULL;
      myD.clock = 0;
      /* the gazebo08 device must be opened before this operation. The image size 
	 of the jde varcolor image is taken from gazebo08 device */
      if (color_name[7].tipo == 1){	  
	  myD.width=camera[7]->width;
	  myD.height=camera[7]->height;
      }
      else if (color_name[7].tipo > 1){
	  myD.width=stereo->width;
	  myD.height=stereo->height;
      }
      myD.img = (char*)malloc(myD.width*myD.height*3*sizeof(char));
      all[num_schemas].id = (int *) &camera_schema_id[7];
      (*(all[num_schemas].id)) = num_schemas;
      strcpy(all[num_schemas].name,(char*)"varcolorD");
      all[num_schemas].run = (runFn) gazebo08_camera7_run;
      all[num_schemas].stop = (stopFn) gazebo08_camera7_stop;     
      all[num_schemas].fps = 0.;
      all[num_schemas].k = 0;
      all[num_schemas].state = slept;
      all[num_schemas].terminate = NULL;
      all[num_schemas].handle = NULL;     
      printf ((char*)"%s schema loaded (id %d)\n", all[num_schemas].name, *(all[num_schemas].id));
      num_schemas++;
      myexport((char*)"varcolorD",(char*)"id",&camera_schema_id[7]);
      myexport((char*)"varcolorD",(char*)"varcolorD",&myD);
      myexport((char*)"varcolorD",(char*)"run",(void *)gazebo08_camera7_run);
      myexport((char*)"varcolorD",(char*)"stop",(void *)gazebo08_camera7_stop);
    }
 
  /* gazebo08 thread creation */
  pthread_mutex_lock (&mymutex);
  state = slept;
  pthread_create (&gazebo08_th, NULL, gazebo08_thread, NULL);
  pthread_mutex_unlock (&mymutex);

  return 0;
}

int
gazebo08_terminate ()
{
  int i;
  
  gazebo08_terminate_command = 1;
  lgz_delete(gz_lib);
  printf ((char*)"Switching gazebo08 off\n");
}


int
gazebo08_ptmotors_run (int father, int *brothers, arbitration fn)
{
  
  if ((serve_ptmotors) && (ptmotors_active == 0))
    {
      ptmotors_active = 1;
      lgz_setDataPtzIface(gz_lib,0.0,0.0);
      pan_angle=0.0;
      tilt_angle=0.0;
      longitude=0.0;
      latitude=0.0;
      put_state (ptmotors_schema_id, winner);
      printf ((char*)"gazebo08: ptz command  run\n");
      all[ptmotors_schema_id].father = father;
      all[ptmotors_schema_id].fps = 0.;
      all[ptmotors_schema_id].k = 0;
      should_driver_restart();
    }
  return 0;
}

int
gazebo08_ptmotors_stop ()
{
  if ((serve_ptmotors) && (ptmotors_active))
    {
      ptmotors_active = 0;
      printf ((char*)"gazebo08: ptz command stop\n");
      put_state (ptmotors_schema_id, slept);
      should_driver_stop(); 
    }
  return 0;
}

int
gazebo08_ptencoders_run (int father, int *brothers, arbitration fn)
{
  
  if ((serve_ptencoders) && (ptencoders_active == 0))
    {
      ptencoders_active = 1;
      put_state (ptencoders_schema_id, winner);
      all[ptencoders_schema_id].father = father;
      all[ptencoders_schema_id].fps = 0.;
      all[ptencoders_schema_id].k = 0;
      should_driver_restart();
    }
  return 0;
}

int
gazebo08_ptencoders_stop ()
{
  if ((serve_ptencoders) && (ptencoders_active))
    {
      ptencoders_active = 0;
      printf ((char*)"gazebo08: ptz encoders stop\n");
      put_state (ptencoders_schema_id, slept);
      should_driver_stop();
    }
  return 0;
}

int
gazebo08_camera0_run (int father, int *brothers, arbitration fn)
{
 
  if ((serve_color[0]==1)&&(color_active[0]==0)) 
     {       
       color_active[0]++;
       if ((all[camera_schema_id[0]].father==GUIHUMAN) ||
	   (all[camera_schema_id[0]].father==SHELLHUMAN))
	 all[camera_schema_id[0]].father = father;
       if(color_active[0]==1)	 
	 {
	   all[camera_schema_id[0]].father = father;
	   all[camera_schema_id[0]].fps = 0.;
	   all[camera_schema_id[0]].k = 0;
	   put_state (camera_schema_id[0], winner);
	   should_driver_restart();
	 }
     }
   return 0;
}

int
gazebo08_camera0_stop ()
{
  if ((serve_color[0]) && (color_active[0]==0))
    {
      color_active[0]--;
      if (color_active[0]<=0)
	{
	  put_state (camera_schema_id[0], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}


int
gazebo08_camera1_run (int father, int *brothers, arbitration fn)
{
  
  if ((serve_color[1]==1)&&(color_active[1]==0)) 
     {
      color_active[1]++;
      if ((all[camera_schema_id[1]].father==GUIHUMAN) ||
	  (all[camera_schema_id[1]].father==SHELLHUMAN))
	all[camera_schema_id[1]].father = father;
      if(color_active[1]==1)
	{
	  all[camera_schema_id[1]].father = father;
	  all[camera_schema_id[1]].fps = 0.;
	  all[camera_schema_id[1]].k = 0;
	  put_state (camera_schema_id[1], winner);
	  should_driver_restart();
	}
     }
  return 0;
}


int
gazebo08_camera1_stop ()
{
  if ((serve_color[1]) && (color_active[1]))
    {
      color_active[1]--;
      if (color_active[1]<=0)
	{
	  put_state (camera_schema_id[1], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}


int
gazebo08_camera2_run (int father, int *brothers, arbitration fn)
{

  if ((serve_color[2]==1)&&(color_active[2]==0)) 
     { 
       color_active[2]++;
       if ((all[camera_schema_id[2]].father==GUIHUMAN) ||
	   (all[camera_schema_id[2]].father==SHELLHUMAN))
	 all[camera_schema_id[2]].father = father;
       if(color_active[2]==1)
	 {
	   all[camera_schema_id[2]].father = father;
	   all[camera_schema_id[2]].fps = 0.;
	   all[camera_schema_id[2]].k = 0;
	   put_state (camera_schema_id[2], winner);
	   should_driver_restart();
	 }
     }
  return 0;
}

int
gazebo08_camera2_stop ()
{
  if ((serve_color[2]) && (color_active[2]))
    {
      color_active[2]--;
      if (color_active[2]<=0)
	{
	  put_state (camera_schema_id[2], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}


int
gazebo08_camera3_run (int father, int *brothers, arbitration fn)
{
 if ((serve_color[3]==1)&&(color_active[3]==0)) 
     {
       color_active[3]++;
      if ((all[camera_schema_id[3]].father==GUIHUMAN) ||
	  (all[camera_schema_id[3]].father==SHELLHUMAN))
	all[camera_schema_id[3]].father = father;
      if(color_active[3]==1)	
	{
	  all[camera_schema_id[3]].father = father;
	  all[camera_schema_id[3]].fps = 0.;
	  all[camera_schema_id[3]].k = 0;
	  put_state (camera_schema_id[3], winner);
	  should_driver_restart();
	}
     }
 return 0;
}



int
gazebo08_camera3_stop ()
{
  if ((serve_color[3]) && (color_active[3]))
    {
      color_active[3]--;
      printf ((char*)"gazebo08: camera 3 stop\n");
      if (color_active[3]<=0)
	{
	  put_state (camera_schema_id[2], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}

int
gazebo08_camera4_run (int father, int *brothers, arbitration fn)
{
  if ((serve_color[4]==1)&&(color_active[4]==0)) 
     {
       color_active[4]++;
      if ((all[camera_schema_id[4]].father==GUIHUMAN) ||
	  (all[camera_schema_id[4]].father==SHELLHUMAN))
	all[camera_schema_id[4]].father = father;
      if(color_active[4]==1)	
	{
	  all[camera_schema_id[4]].father = father;
	  all[camera_schema_id[4]].fps = 0.;
	  all[camera_schema_id[4]].k = 0;
	  put_state (camera_schema_id[4], winner);
	  should_driver_restart();	 
	}
     }
  return 0;
}


int
gazebo08_camera4_stop ()
{
  if ((serve_color[4]) && (color_active[4]))
    {
      color_active[4]--;
      /* printf ((char*)"gazebo08: camera 4 stop\n");*/
      if (color_active[4]<=0)
	{
	  put_state (camera_schema_id[4], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}

int
gazebo08_camera5_run (int father, int *brothers, arbitration fn)
{
  if ((serve_color[5]==1)&&(color_active[5]==0)) 
     {
       color_active[5]++;
      if ((all[camera_schema_id[5]].father==GUIHUMAN) ||
	  (all[camera_schema_id[5]].father==SHELLHUMAN))
	all[camera_schema_id[5]].father = father;
      if(color_active[5]==1)	
	{
	  all[camera_schema_id[5]].father = father;
	  all[camera_schema_id[5]].fps = 0.;
	  all[camera_schema_id[5]].k = 0;
	  put_state (camera_schema_id[5], winner);
	  should_driver_restart();	 
	}
     }
  return 0;
}


int
gazebo08_camera5_stop ()
{
  if ((serve_color[5]) && (color_active[5]))
    {
      color_active[5]--;
      /* printf ((char*)"gazebo08: camera 5 stop\n");*/
      if (color_active[5]<=0)
	{
	  put_state (camera_schema_id[5], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}

int
gazebo08_camera6_run (int father, int *brothers, arbitration fn)
{
  if ((serve_color[6]==1)&&(color_active[6]==0)) 
     {
       color_active[6]++;
      if ((all[camera_schema_id[6]].father==GUIHUMAN) ||
	  (all[camera_schema_id[6]].father==SHELLHUMAN))
	all[camera_schema_id[6]].father = father;
      if(color_active[6]==1)	
	{
	  all[camera_schema_id[6]].father = father;
	  all[camera_schema_id[6]].fps = 0.;
	  all[camera_schema_id[6]].k = 0;
	  put_state (camera_schema_id[6], winner);
	  should_driver_restart();	 
	}
     }
  return 0;
}


int
gazebo08_camera6_stop ()
{
  if ((serve_color[6]) && (color_active[6]))
    {
      color_active[6]--;
      /* printf ((char*)"gazebo08: camera 6 stop\n");*/
      if (color_active[6]<=0)
	{
	  put_state (camera_schema_id[6], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}

int
gazebo08_camera7_run (int father, int *brothers, arbitration fn)
{
  if ((serve_color[7]==1)&&(color_active[7]==0)) 
     {
       color_active[7]++;
      if ((all[camera_schema_id[7]].father==GUIHUMAN) ||
	  (all[camera_schema_id[7]].father==SHELLHUMAN))
	all[camera_schema_id[7]].father = father;
      if(color_active[7]==1)	
	{
	  all[camera_schema_id[7]].father = father;
	  all[camera_schema_id[7]].fps = 0.;
	  all[camera_schema_id[7]].k = 0;
	  put_state (camera_schema_id[7], winner);
	  should_driver_restart();	 
	}
     }
  return 0;
}


int
gazebo08_camera7_stop ()
{
  if ((serve_color[7]) && (color_active[7]))
    {
      color_active[7]--;
      /* printf ((char*)"gazebo08: camera 7 stop\n");*/
      if (color_active[7]<=0)
	{
	  put_state (camera_schema_id[7], slept);
	  should_driver_stop();     
	}
    }
  return 0;
}



int
gazebo08_encoders_run (int father, int *brothers, arbitration fn)
{
  if ((serve_encoders) && (encoders_active == 0))
    {
      encoders_active = 1;
      put_state (encoders_schema_id, winner);
      printf ((char*)"gazebo08: encoders run\n");
      all[encoders_schema_id].father = father;
      all[encoders_schema_id].fps = 0.;
      all[encoders_schema_id].k = 0;
      should_driver_restart();	  
    }
  return 0;
}

int
gazebo08_encoders_stop ()
{

  if ((serve_encoders) && (encoders_active))
    {
      encoders_active = 0;
      put_state (encoders_schema_id, slept);
      printf ((char*)"gazebo08: encoders stop\n");
      should_driver_stop();  
    }
  return 0;
}

int
gazebo08_motors_run (int father, int *brothers, arbitration fn)
{
  if ((serve_motors) && (motors_active == 0))
    {
      put_state (motors_schema_id, winner);
      motors_active = 1;
      printf ((char*)"gazebo08: motors run\n");
      all[motors_schema_id].father = father;
      all[motors_schema_id].fps = 0.;
      all[motors_schema_id].k = 0;
      should_driver_restart();	  
    }
  return 0;
}

int
gazebo08_motors_stop ()
{

  if ((serve_motors) && (motors_active))
    {
      int v_sp = 0;
      int w_sp = 0;
      motors_active = 0;

      /*Velocidad a cero */
      lgz_setDataPosIface(gz_lib,v_sp,w_sp);

      printf ((char*)"gazebo08: motors stop\n");
      put_state (motors_schema_id, slept);
      should_driver_stop();  
    }
  return 0;
}

int
gazebo08_laser_run (int father, int *brothers, arbitration fn)
{
  if ((serve_laser) && (laser_active == 0))
    {
      laser_active = 1;
      put_state (laser_schema_id, winner);
      printf ((char*)"gazebo08: laser run\n");
      all[laser_schema_id].father = father;
      all[laser_schema_id].fps = 0.;
      all[laser_schema_id].k = 0;
      should_driver_restart();	  
    }
  return 0;
}

int
gazebo08_laser_stop ()
{

  if ((serve_laser) && (laser_active))
    {
      laser_active = 0;
      put_state (laser_schema_id, slept);
      printf ((char*)"gazebo08: laser stop\n");
      should_driver_stop();  
    }
  return 0;
}

void
gazebo08_motors_iteration ()
{

  double v_double, w_double;
  float x,y,yaw;
  speedcounter (motors_schema_id);

  //tomamos la velocidad actual
  lgz_getDataPosIface(gz_lib,&x,&y,&yaw);
  v_double = x;
  w_double = yaw;

  //jde trabaja en  mm/s, gazebo08 en m/s 
  // w esta en grados/s , gazebo08 trata radianes/s
  lgz_setDataPosIface(gz_lib,(v/1000),w * DEGTORAD);
}

void *
gazebo08_thread (void *not_used)
{
  struct timeval t;
  unsigned long now,next;
  static unsigned long lastmotor = 0;
  static unsigned long lastiteration = 0;
  int i;

  printf ((char*)"gazebo08: gazebo08 thread started up\n");

  do
    {
      pthread_mutex_lock (&mymutex);
      if (state == slept)
	{
	  printf ((char*)"gazebo08: gazebo08 thread in sleep mode\n");
	  pthread_cond_wait (&condition, &mymutex);
	  printf ((char*)"gazebo08: gazebo08 thread woke up\n");
	  pthread_mutex_unlock (&mymutex);
	}
      else
	{
	  pthread_mutex_unlock (&mymutex);
	  gettimeofday (&t, NULL);
	  now = t.tv_sec * 1000000 + t.tv_usec;
	  lastiteration = now;

	  /* sending motors command */
	  if ((serve_motors) && (motors_active))
	    {
	      if ((now - lastmotor) > gazebo08_COMMAND_CYCLE * 1000)
		{
		  lastmotor = now;
		  gazebo08_motors_iteration ();
		}
	    }

	  /*Actualizamos */
	  if (laser_active)
	    gazebo08_laser_callback ();

	  if (ptmotors_active)
		gazebo08_ptmotors_callback();

	  if (ptencoders_active)
		gazebo08_ptencoders_callback();

	  for(i=0;i<MAXCAM;i++)
	    {
	      if ((serve_color[i]) && (color_active[i]))
		gazebo08_camera_callback (i);
	    }

	  if (encoders_active)
	    gazebo08_encoders_callback ();

	 /* to control the iteration time of this driver */
	  gettimeofday (&t, NULL);
	  now = t.tv_sec * 1000000 + t.tv_usec;
	  next=lastiteration+(long)gazebo08_CYCLE*1000;
	  if (next>(5000+now))
	    {
	      usleep(next-now-5000);
	    }
	}
    }
  while (gazebo08_terminate_command == 0);
  pthread_exit (0);
}

void
gazebo08_camera_callback (int camnum)
{
  char *destination=NULL;
  char *origen=NULL;
  float *origendisparity=NULL;
  int myrows=0,mycolumns=0;
  int i=0;
  float scalefactor = 255;
  /* 4 is a ad hoc scale factor to avoid dark disparity images */

  speedcounter (camera_schema_id[camnum]);
  switch (camnum)
    {
    case 0:
      destination = colorA;
      myrows=SIFNTSC_ROWS;
      mycolumns=SIFNTSC_COLUMNS;
      break;
    case 1:
      destination = colorB;
      myrows=SIFNTSC_ROWS;
      mycolumns=SIFNTSC_COLUMNS;
      break;
    case 2:
      destination = colorC;
      myrows=SIFNTSC_ROWS;
      mycolumns=SIFNTSC_COLUMNS;
      break;
    case 3:
      destination = colorD;
      myrows=SIFNTSC_ROWS;
      mycolumns=SIFNTSC_COLUMNS;
      break;
    case 4:
      destination = myA.img;
      myrows=myA.height;
      mycolumns=myA.width;
      break;
    case 5:
      destination = myB.img;
      myrows=myB.height;
      mycolumns=myB.width;
      break;
    case 6:
      destination = myC.img;
      myrows=myC.height;
      mycolumns=myC.width;
      break;
    case 7:
      destination = myD.img;
      myrows=myD.height;
      mycolumns=myD.width;
      break;
    }

  if(color_name[camnum].tipo==1){
      lgz_getDataCameraIface(gz_lib,camera[camnum]);
      origen = (char*) camera[camnum]->data;
  }
  else{
      lgz_getDataStereoIface(gz_lib,stereo);
      switch (color_name[camnum].tipo){
	    case 2:
	      origendisparity = stereo->depth_left;
	      // left_disparity contains float values 
	      break;
	    case 3:
	      origendisparity = stereo->depth_right;
	      // right_disparity contains float values 
	      break;
      }
  }

  if ((color_name[camnum].tipo)==1){
	  /* parece que un dispositivo es RGB y el otro BGR 
		 por tanto damos el cambiazo aqui */
	  for (i = 0; i < myrows*mycolumns*3; i += 3){
		  destination[i] = origen[i + 2];
		  destination[i + 1] = origen[i + 1];
		  destination[i + 2] = origen[i];
	  }
  }
  else{
         /* las imagenes de profundidad solo tienen un canal.
                las representamos en niveles de gris */
         for (i = 0; i < myrows*mycolumns; i++)
	 {
		  destination[3*i] = (char)(int)(scalefactor*origendisparity[i]);
		  destination[3*i + 1] = (char)(int)(scalefactor*origendisparity[i]);
		  destination[3*i + 2] = (char)(int)(scalefactor*origendisparity[i]);
		  //printf("disp value= %0.4f\n",origendisparity[i]);
	 }
  }
   
}


void
gazebo08_ptmotors_callback(){
  	speedcounter (ptmotors_schema_id);
        static int d=0;

	if(serve_ptmotors){
		/* enforce the position limits */
		/* ignores the pan speed and tilt speed values */
		if (longitude > max_pan)
		  longitude = max_pan;
		else if (longitude < min_pan)
		  longitude = min_pan;

		if (latitude > max_tilt)
		  latitude = max_tilt;
		else if (latitude < min_tilt)
		  latitude = min_tilt;

		/* pan in gazebo08's pantilt is the opposite to pan in directed perception pantilt unit.  */
		/* tilt in gazebo08's pantilt is the opposite to pan in directed perception pantilt unit.  */
		lgz_setDataPtzIface(gz_lib,(double)longitude * DEGTORAD,(double)latitude * DEGTORAD);
	
		/*printf( "Pan Tilt motors TO gazebo08: \n longitude: %f latitude:%f\n",longitude,latitude);*/
	}
}

void
gazebo08_ptencoders_callback(){
	double pan,tilt;
  	speedcounter (ptencoders_schema_id);

	if(serve_ptencoders){
		/* pan in gazebo08's pantilt is the opposite to pan in directed perception pantilt unit.  */
		/* tilt in gazebo08's pantilt is the opposite to pan in directed perception pantilt unit.  */
		lgz_getDataPtzIface(gz_lib,&pan,&tilt);
		pan_angle= pan * RADTODEG;
		tilt_angle= tilt * RADTODEG;
		//printf( "Pan Tilt encoders from gazebo08: \n pan_angle: %f:tilt_angle:%f\n",pan_angle,tilt_angle);
	}
}

void
gazebo08_laser_callback ()
{
/* 
 * gazebo08 ofrece un numero de medidas configurable
 * Mapeamos las medidas de gazebo08 en las de jde
 * una a una cuando el numero de medidas coincide
 * una a varias o viceversa cuando el n�mero de medidas difiere 
 */

  int cont = 0;
  int cont2;
  double ranges[1024];
  int range_count;
  double relation; //(gazebo08 ray number / jde rays number)

  speedcounter(laser_schema_id);

  lgz_getDataLaserIface(gz_lib,ranges,&range_count);
  relation= (float)range_count /(float)NUM_LASER ;
  
  for (cont = 0; cont < NUM_LASER &&
       (relation * cont) < range_count; cont++)
    {
      cont2 = rint(relation * cont);
      jde_laser[cont] = (int) (ranges[cont2] * 1000);
	
    }
}

void
gazebo08_encoders_callback ()
{

  float robotx, roboty, robottheta;
  float x,y,yaw;
  speedcounter(encoders_schema_id);

  lgz_getDataPosIface(gz_lib,&x,&y,&yaw);
  robotx =
    x * 1000 * (float) cos (DEGTORAD * correcting_theta) -
    y * 1000 * (float) sin (DEGTORAD * correcting_theta) +
    correcting_x;
  roboty =
    y * 1000 * (float) cos (DEGTORAD * correcting_theta) +
    x * 1000 * (float) sin (DEGTORAD * correcting_theta) +
    correcting_y;
  robottheta = (yaw * RADTODEG) + correcting_theta;

  if (robottheta <= 0) robottheta = robottheta + 360;
  else if (robottheta > 360) robottheta = robottheta - 360;

  jde_robot[0] = robotx;
  jde_robot[1] = roboty;
  jde_robot[2] = robottheta * DEGTORAD;
  jde_robot[3] = cos (jde_robot[2]);
  jde_robot[4] = sin (jde_robot[2]);
}

int
gazebo08_parseconf (char *configfile)
{
  FILE *conf = fopen (configfile,(char*)"r");
  char cpLinea[MAX_BUFFER],word3[MAX_BUFFER],word4[MAX_BUFFER],word5[MAX_BUFFER];
  char *pLine1, *pLine2;
  int ItIsAGazeboLine = 0, i, colors;
  char *camera_token[] = {(char*) "colorA",(char*)"colorB",(char*)"colorC",(char*)"colorD",(char*)"varcolorA",(char*)"varcolorB",(char*)"varcolorC",(char*)"varcolorD" };


  if (!conf)
    return (-1);
  while (!feof (conf))
    {
      fgets (cpLinea, 160, conf);
      i = 0;
      while (isspace (cpLinea[i++]));
      if (cpLinea[i - 1] == '#')
	continue;
      if (strstr (cpLinea,(char*)"gazebo08"))
	{
	  ItIsAGazeboLine = 1;
	  continue;
	}
      if (!ItIsAGazeboLine)
	continue;
      /* puts (cpLinea);*/
      /*tenemos lineas de nuestro driver */
      if (strstr (cpLinea,(char*)"end_driver"))
	break;

      for (colors = MAXCAM-1; colors >=0; colors--)
	{
	  pLine1 = strstr (cpLinea, camera_token[colors]);
	  if (pLine1 )
	    {
	      pLine2 = strstr (pLine1,(char*)" ");
	      for (i = 0; isspace (pLine2[0]); pLine2++);	//salto blancos
	      
	      if (strstr (pLine2,(char*)"left"))
		{		/*Stereo left dispsrity */
		  pLine1 = strstr (pLine2,(char*)" ");
		  pLine1[0] = (char) 0;
		  /* printf ((char*)"Nombre del estero left:%s:\n", pLine2); */
		  serve_color[colors]=1;
		  strcpy (color_name[colors].name, pLine2);
		  color_name[colors].tipo = 2;	/*left disparity */
		  puts(color_name[colors].name);
		  break;
		}
	      
	      else if (strstr (pLine2,(char*)"right"))
		{		/*Stereo right dispsrity */
		  pLine1 = strstr (pLine2,(char*)" ");
		  pLine1[0] = (char) 0;
		  serve_color[colors]=1;
		  strcpy (color_name[colors].name, pLine2);	/*stereo id */
		  /* printf ((char*)"Nombre del estero right:%s:\n", pLine2); */
		  color_name[colors].tipo = 3;	/*right disparity */
		  puts(color_name[colors].name);
		  break;
		}
	      else
		{		/*Se trata de una camara mono */
		  pLine1 = strstr (pLine2,(char*)"\n");
		  pLine1[0] = (char) 0;
		  serve_color[colors]=1;
		  strcpy(color_name[colors].name, pLine2);	/*camera id */
		  color_name[colors].tipo = 1;	/*camara mono */
		  puts(color_name[colors].name);
		  break;
		}
	    }
	}			/*endfor */
      
      pLine2=strstr(cpLinea,(char*)" laser ");
      if (pLine2)
	{			//Tomo el surtidor del laser
	  pLine2 += 7; //Me salto " laser ";
	  for (i = 0; isspace (pLine2[0]); pLine2++);	//salto blancos
	  pLine1 = strstr (pLine2,(char*)"\n");
	  pLine1[0] = (char) 0;
	  serve_laser=1;
	  strncpy (laser_name, pLine2, MAX_MODEL_ID);
	}
      
      pLine2 = strstr (cpLinea,(char*)" motors ");
      if (pLine2)
	{			//Tomo el surtidor de motores
	  pLine2 += 8;
	  for (; isspace (pLine2[0]); pLine2++);	//salto blancos
	  pLine1 = strstr (pLine2,(char*)"\n");
	  pLine1[0] = (char) 0;
	  serve_motors=1;
	  strncpy (motors_name, pLine2, MAX_MODEL_ID);
	}
      
      pLine2 = strstr(cpLinea,(char*)" encoders ");
      if (pLine2)
	{			//Tomo el surtidor de encoders
	  pLine2 += 10;
	  for (i = 0; isspace (pLine2[0]); pLine2++);	//salto blancos
	  pLine1 = strstr (pLine2,(char*)"\n");
	  pLine1[0] = (char) 0;
	  serve_encoders=1;
	  strncpy (position_name, pLine2, MAX_MODEL_ID);
	}
      
      pLine2=strstr(cpLinea,(char*)" pantilt ");
      if (pLine2)
	{			//Tomo el surtidor de ptz
	  pLine2 += 9;
	  for (i = 0; isspace (pLine2[0]); pLine2++);	//salto blancos
	  pLine1 = strstr (pLine2,(char*)"\n");
	  pLine1[0] = (char) 0;
	  serve_ptencoders=1;
	  serve_ptmotors=1;
	  strncpy (ptz_name, pLine2, MAX_MODEL_ID);
	}

      pLine2 = strstr(cpLinea,(char*)"initial_position ");
      if(pLine2) { //Tomo el surtidor de initial_position
	pLine2 += 17;
	for (i = 0; isspace(pLine2[0]); pLine2++); //salto blancos
	printf((char*)"XXX El nombre para initial_position :%s:\n",pLine2);
	if(sscanf(pLine2,(char*)"%s %s %s",word3,word4,word5)>2){
	  correcting_x=(float)atof(word3);
	  correcting_y=(float)atof(word4);
	  correcting_theta=(float)atof(word5);
	  printf((char*)"gazebo08: correcting x=%.1f(mm)  y=%.1f(mm)  theta=%.1f(deg).\n",
		 correcting_x,correcting_y,correcting_theta);
	  printf((char*)"gazebo08: make sure this is the initial position in the world file provided to gazebo08 simulator.\n");
	}else
	  printf((char*)"gazebo08: wrong initial_position line in the configuration file\n");
	pLine1 = strstr(pLine2,(char*)"\n");
	pLine1[0] = (char) 0;
      }
    }
  
  fclose (conf);
  return 0;
}
