#include "libgazebo.h"

   //Constructor
   libgazebo::libgazebo(){
   	client= new gazebo::Client();
	simIface= new gazebo::SimulationIface();
        posIface=NULL;
	laserIface=NULL;
	for(int i=0; i<MAXCAM; i++)
		cameraIfaces[i]=NULL;
	countcam=0;
   }

   // libgazebo init
   int libgazebo::init(int server_id,int client_id){
	  try
	  {
	    client->ConnectWait(server_id, GZ_CLIENT_ID_USER_FIRST);
            simIface->Open(client, "default");
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to connect\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo terminate
   int libgazebo::terminate(){
	  try
	  {
	    if(posIface!=NULL){
		posIface->Close();
		delete posIface;
	    }
	    if(laserIface!=NULL){
		laserIface->Close();
		delete laserIface;
	    }

	    client->Disconnect();
	    delete client;
	    delete simIface;
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to disconnect\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo open position interface
   int libgazebo::openPosIface(char *id){
	  try
	  {
	    posIface= new gazebo::PositionIface();
	    posIface->Open(client, id);
	    posIface->Lock(1);
	    posIface->data->cmdEnableMotors = 1;
	    posIface->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to open the position interface\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo getting position data
   int libgazebo::getDataPosIface(float *x,float* y,float* yaw){
	  try
	  {
    	    posIface->Lock(1);
	    *x=posIface->data->pose.pos.x;
	    *y=posIface->data->pose.pos.y;
	    *yaw=posIface->data->pose.yaw;
            posIface->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to get position data\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo setting position data
   int libgazebo::setDataPosIface(float v,float w){
	  try
	  {
    	    posIface->Lock(1);
            posIface->data->cmdVelocity.pos.x = v;
            posIface->data->cmdVelocity.yaw = w;
            posIface->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to set position data\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo open laser interface
   int libgazebo::openLaserIface(char *id){
	  try
	  {
	    laserIface= new gazebo::LaserIface();
	    laserIface->Open(client, id);
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to open the laser interface\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo getting laser data
   int libgazebo::getDataLaserIface(double *ranges,int *range_count){
	  try
	  {
    	    laserIface->Lock(1);
	    *range_count=laserIface->data->range_count;
	    for(int i=0; i<*range_count; i++){
		ranges[i]=laserIface->data->ranges[i];
	    }
            laserIface->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to get laser data\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo open camera interface
   int libgazebo::openCameraIface(char *id,lgz_camera* cam){
	  try
	  {
	    cameraIfaces[countcam]= new gazebo::CameraIface();
	    cameraIfaces[countcam]->Open(client, id);
	    cam->width=cameraIfaces[countcam]->data->width;
	    cam->height=cameraIfaces[countcam]->data->height;
	    if(cam->width==0 || cam->height==0){
	        std::cout << "Gazebo error: assertion 'image width > 0' failed for camera interface\n";
		return -1;
            }
	    cam->data=(unsigned char*)malloc(cam->width*cam->height*3*sizeof(char));
	    cam->id=countcam;
	    countcam++;
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to open the camera interface\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo getting camera data
   int libgazebo::getDataCameraIface(lgz_camera *camera){
	  try
	  {
    	    cameraIfaces[camera->id]->Lock(1);
	    for(int i=0; i<camera->width*camera->height*3; i++){
		camera->data[i]=cameraIfaces[camera->id]->data->image[i];
	    }
            cameraIfaces[camera->id]->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to get camera data\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo open stereo camera interface
   int libgazebo::openStereoIface(char *id,lgz_stereo* stereo){
	  try
	  {
	    stereoIface= new gazebo::StereoCameraIface();
	    stereoIface->Open(client, id);
	    stereo->width=stereoIface->data->width;
	    stereo->height=stereoIface->data->height;
	    if(stereo->width==0 || stereo->height==0){
	        std::cout << "Gazebo error: assertion 'image width > 0' failed for stereo interface\n";
		return -1;
            }
	    stereo->depth_left=(float*)malloc(stereo->width*stereo->height*sizeof(float));
	    stereo->depth_right=(float*)malloc(stereo->width*stereo->height*sizeof(float));
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to open the stereo interface\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo getting laser data
   int libgazebo::getDataStereoIface(lgz_stereo* stereo){
	  try
	  {
    	    stereoIface->Lock(1);
	    for(int i=0; i<stereo->width*stereo->height; i++)
			stereo->depth_left[i]=stereoIface->data->left_depth[i];

	    for(int i=0; i<stereo->width*stereo->height; i++)
			stereo->depth_right[i]=stereoIface->data->right_depth[i];
            stereoIface->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to get stereo data\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo open ptz interface
   int libgazebo::openPtzIface(char *id){
	  try
	  {
	    ptzIface= new gazebo::PTZIface();
	    ptzIface->Open(client, id);
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to open the position interface\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo getting ptz data
   int libgazebo::getDataPtzIface(double *pan,double* tilt){
	  try
	  {
    	    ptzIface->Lock(1);
	    *pan=(double)ptzIface->data->pan;
	    *tilt=(double)ptzIface->data->tilt;
            ptzIface->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to get ptz data\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

   // libgazebo setting ptz data
   int libgazebo::setDataPtzIface(double pan,double tilt){
	  try
	  {
	    double p,t;
    	    ptzIface->Lock(1);
            ptzIface->data->cmd_pan = pan;
            ptzIface->data->cmd_tilt = tilt;
	    p=(double)ptzIface->data->pan;
	    t=(double)ptzIface->data->tilt;
            ptzIface->Unlock();
	  }
	  catch (gazebo::GazeboError e)
	  {
	    std::cout << "Gazebo error: Unable to set ptz data\n" << e << "\n";
	    return -1;
	  }
	  return 0;
   }

//C EXPORTED FUNCTIONS
   libgazebo* lgz_create(){
	return new libgazebo();
   }

   void lgz_delete(libgazebo* self){
	self->terminate();
	delete self;
   }

   int lgz_init(libgazebo* self,int server_id,int client_id){
	return self->init(server_id,client_id);
   }

   int lgz_openPosIface(libgazebo* self,char *id){
	return self->openPosIface(id);	
   }

   int lgz_getDataPosIface(libgazebo* self,float *x,float *y,float *yaw){
	return self->getDataPosIface(x,y,yaw);
   }

   int lgz_setDataPosIface(libgazebo* self,float v,float w){
	return self->setDataPosIface(v,w);
   }

   int lgz_openLaserIface(libgazebo* self,char *id){
	return self->openLaserIface(id);	
   }

   int lgz_getDataLaserIface(libgazebo* self,double *ranges,int *range_count){
	return self->getDataLaserIface(ranges,range_count);
   }

   int lgz_openCameraIface(libgazebo* self,char *id,lgz_camera* camera){
	return self->openCameraIface(id,camera);	
   }

   int lgz_getDataCameraIface(libgazebo* self,lgz_camera* camera){
	return self->getDataCameraIface(camera);
   }

   int lgz_openStereoIface(libgazebo* self,char *id,lgz_stereo* stereo){
	return self->openStereoIface(id,stereo);	
   }

   int lgz_getDataStereoIface(libgazebo* self,lgz_stereo* stereo){
	return self->getDataStereoIface(stereo);
   }

   int lgz_openPtzIface(libgazebo* self,char *id){
	return self->openPtzIface(id);	
   }

   int lgz_getDataPtzIface(libgazebo* self,double *pan,double *tilt){
	return self->getDataPtzIface(pan,tilt);
   }

   int lgz_setDataPtzIface(libgazebo* self,double pan,double tilt){
	return self->setDataPtzIface(pan,tilt);
   }
