#ifndef libgazebo_H
#define libgazebo_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gazebo/gazebo.h>
#include <gazebo/GazeboError.hh>
#include <camera/StereoCameraSensor.hh>
#include <ctype.h>

#define MAXCAM 8

/* mono camera */
typedef struct camera{
	   unsigned char* data;
	   int width;
	   int height;
	   int id;
} lgz_camera;

/* stereo camera */
typedef struct stereo{
	   float* depth_left;
	   float* depth_right;
	   int width;
	   int height;
} lgz_stereo;

/* libgazebo class */
class libgazebo{
   private:
   	   gazebo::Client *client;
	   gazebo::SimulationIface *simIface;
	   gazebo::PositionIface *posIface;
	   gazebo::LaserIface *laserIface;
	   gazebo::CameraIface *cameraIfaces[MAXCAM];
	   gazebo::StereoCameraIface *stereoIface;
           gazebo::StereoCameraSensor *stereoSensor;
           gazebo::PTZIface *ptzIface;
	   int countcam;

   public:
	   //Constructor
	   libgazebo();

	   //Methods
	   int init(int,int);
	   int terminate();

	   int openPosIface(char*);
	   int getDataPosIface(float*,float*,float*);
	   int setDataPosIface(float,float);

	   int openLaserIface(char*);
	   int getDataLaserIface(double*,int*);

	   int openCameraIface(char*,lgz_camera*);
	   int getDataCameraIface(lgz_camera*);

	   int openStereoIface(char*,lgz_stereo*);
	   int getDataStereoIface(lgz_stereo*);

	   int openPtzIface(char*);
	   int getDataPtzIface(double*,double*);
	   int setDataPtzIface(double,double);
};


//C EXPORTED FUNCTIONS
extern "C" libgazebo* lgz_create();
extern "C" void lgz_delete(libgazebo* self);
extern "C" int lgz_init(libgazebo* self,int server_id,int client_id);

extern "C" int lgz_openPosIface(libgazebo* self,char *id);
extern "C" int lgz_getDataPosIface(libgazebo* self,float *x,float *y,float *yaw);
extern "C" int lgz_setDataPosIface(libgazebo* self,float v,float w);

extern "C" int lgz_openLaserIface(libgazebo* self,char *id);
extern "C" int lgz_getDataLaserIface(libgazebo* self,double *ranges,int* range_count);

extern "C" int lgz_openCameraIface(libgazebo* self,char *id,lgz_camera *camera);
extern "C" int lgz_getDataCameraIface(libgazebo* self,lgz_camera *camera);

extern "C" int lgz_openStereoIface(libgazebo* self,char *id,lgz_stereo* stereo);
extern "C" int lgz_getDataStereoIface(libgazebo* self,lgz_stereo* stereo);

extern "C" int lgz_openPtzIface(libgazebo* self,char *id);
extern "C" int lgz_getDataPtzIface(libgazebo* self,double *pan,double *tilt);
extern "C" int lgz_setDataPtzIface(libgazebo* self,double pan,double tilt);

#endif
